// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_crud_bloc/src/crud/cubit/crud_cubit.dart';
import 'package:wyatt_crud_bloc/src/models/model.dart';

class CrudBuilder<T extends Model<Object?, T>> extends StatelessWidget {
  const CrudBuilder({
    Key? key,
    this.onIdle,
    required this.onLoading,
    required this.onError,
    required this.onSuccess,
  }) : super(key: key);

  final Widget Function(BuildContext, CrudState<T>)? onIdle;
  final Widget Function(BuildContext, CrudState<T>) onLoading;
  final Widget Function(BuildContext, CrudState<T>) onError;
  final Widget Function(BuildContext, CrudState<T>) onSuccess;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<CrudCubit<T>, CrudState<T>>(
      builder: (context, state) {
        switch (state.status) {
          case CrudStatus.idle:
            return onIdle?.call(context, state) ?? onError.call(context, state);
          case CrudStatus.loading:
            return onLoading.call(context, state);
          case CrudStatus.failure:
            return onError.call(context, state);
          case CrudStatus.success:
            return onSuccess.call(context, state);
        }
      },
    );
  }
}
